package com.sillycycle.bagleyd.cubes;

/*
 * @(#)Cubes.java
 *
 * Copyright 1994 - 2022  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Cursor;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import javax.swing.JFrame;

/**
 * The <code>Cubes</code> class extends the
 * <code>CubesCanvas</code> for handling the responses to the user
 * for the <code>CubesFrame</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/puzzles.html
 */

public class Cubes extends CubesCanvas implements MouseWheelListener,
		MouseListener, KeyListener {
	private static final long serialVersionUID = 42L;

	public Cubes(JFrame frame,
			int sizex, int sizey, int sizez, int base,
			Color blockc,
			Color fg, Color bg, int delay) {
		this.frame = frame;
		this.blocks.x = sizex; this.blocks.y = sizey; this.blocks.z = sizez;
		this.base = base;
		this.blockColor = blockc;
		this.foreground = fg; this.background = bg;
		this.delay = delay;
		numSlices = ((delay < MAX_SLICES) ? delay + 1 : MAX_SLICES);
		firstPaint = true;
		this.addMouseListener(this);
		this.addMouseWheelListener(this);
		this.addKeyListener(this);
		setCursor(new Cursor(Cursor.HAND_CURSOR));
	}

	public boolean processKey(char key, boolean control) {
		switch (key) {
		case '>':
		case '.':
			speedUpPuzzle();
			return true;
		case '<':
		case ',':
			slowDownPuzzle();
			return true;
		case '@':
			((CubesFrame) frame).toggleSound();
			return true;
		case '?':
			((CubesFrame) frame).descriptionHelp();
			return true;
		case '!':
			((CubesFrame) frame).featuresHelp();
			return true;
		case '^':
			((CubesFrame) frame).referencesHelp();
			return true;
		case 'a':
		case 'A':
			((CubesFrame) frame).aboutHelp();
			return true;
		case '*':
			((CubesFrame) frame).showBuffer();
			return true;
		default:
		}
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			return false;
		}
		switch (key) {
		case 'o':
		case 'O':
		case '/':
		case '0':
			movePuzzle(OUTWARDS, control);
			break;
		case '8':
			movePuzzle(TOP, control);
			break;
		case '4':
			movePuzzle(LEFT, control);
			break;
		case 'i':
		case 'I':
		case '5':
			movePuzzle(INWARDS, control);
			break;
		case '6':
			movePuzzle(RIGHT, control);
			break;
		case '2':
			movePuzzle(BOTTOM, control);
			break;
		case 'g':
			getPuzzle();
			break;
		case 'w':
			writePuzzle();
			break;
		case 'G':
			getPuzzleFile();
			break;
		case 'W':
			writePuzzleFile();
			break;
		case 'u':
		case 'U':
			undoPuzzle();
			break;
		case 'r':
		case 'R':
			redoPuzzle();
			break;
		case 'c':
		case 'C':
			clearPuzzle();
			break;
		case 'z':
		case 'Z':
			randomizePuzzle();
			break;
		case 's':
		case 'S':
			solvePuzzle();
			break;
		case 'n':
		case 'N':
			numberPuzzle();
			break;
		case 'q':
		case 'Q':
		case 'x':
		case 'X':
		//case '\03': /* Control-C */
			((CubesFrame) frame).quit();
			break;
		default:
			return false;
		}
		return true;
	}

	@Override
	public void mousePressed(MouseEvent event) {
		int modifiers = event.getModifiersEx();

		if (movePaint || scanPaint || stackPaint || mouseDown)
			return;
		if (solve != null && solve.getFlag()) {
			solve.setFlag(false);
			return;
		}
		if (((modifiers & InputEvent.META_DOWN_MASK) == 0) &&
				((modifiers & InputEvent.ALT_DOWN_MASK) == 0)) {
			if (randomPaints == 0) {
				selectPuzzle(event.getX(), event.getY());
				mouseDown = true;
			}
		} else {
			if ((event.getClickCount() & 1) == 1)
				clearWithQueryPuzzle();
			else
				clearPuzzle();
		}
	}

	@Override
	public void mouseClicked(MouseEvent event) {
		/* Method required but mousePressed does work */
	}

	@Override
	public void mouseReleased(MouseEvent event) {
		if (mouseDown) { // Do not care which mouse button
			releasePuzzle();
			mouseDown = false;
		}
	}

	@Override
	public void mouseEntered(MouseEvent event) {
		focus = true;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseExited(MouseEvent event) {
		if (mouseDown && currentPosition >= 0) {
			currentSpace = -1;
			releasePaint = true;
		}
		focus = false;
		framePaint = true;
		mouseDown = false;
		repaint();
	}

	@Override
	public void mouseWheelMoved(MouseWheelEvent event) {
		int notches = event.getWheelRotation();
		boolean control = ((event.getModifiersEx() &
			InputEvent.CTRL_DOWN_MASK) != 0);

		processKey(((notches < 0) ? '8' : '2'), control);
	}

	@Override
	public void keyPressed(KeyEvent event) {
		boolean control = ((event.getModifiersEx() &
			InputEvent.CTRL_DOWN_MASK) != 0);

		switch (event.getKeyCode()) {
		case KeyEvent.VK_ESCAPE:
			((CubesFrame) frame).shuffleDown();
			return;
		case KeyEvent.VK_F11:
		case KeyEvent.VK_DIVIDE:
			processKey('/', control);
			return;
		case KeyEvent.VK_UP:
		case KeyEvent.VK_KP_UP:
		case KeyEvent.VK_NUMPAD8:
			processKey('8', control);
			return;
		case KeyEvent.VK_LEFT:
		case KeyEvent.VK_KP_LEFT:
		case KeyEvent.VK_NUMPAD4:
			processKey('4', control);
			return;
		case KeyEvent.VK_F12:
		case KeyEvent.VK_CLEAR:
		case KeyEvent.VK_NUMPAD5:
			processKey('5', control);
			return;
		case KeyEvent.VK_RIGHT:
		case KeyEvent.VK_KP_RIGHT:
		case KeyEvent.VK_NUMPAD6:
			processKey('6', control);
			return;
		case KeyEvent.VK_DOWN:
		case KeyEvent.VK_KP_DOWN:
		case KeyEvent.VK_NUMPAD2:
			processKey('2', control);
			return;
		default:
			processKey(event.getKeyChar(), control);
		}
	}

	@Override
	public void keyReleased(KeyEvent event) {
		/* Method required but keyPressed does work */
	}

	@Override
	public void keyTyped(KeyEvent event) {
		/* Method required but keyPressed does work */
	}
}
